# -*- coding: utf-8 -*-

"""
.. module:: xml_metafor_cim_v1.__init__.py

   :license: GPL / CeCILL
   :platform: Unix, Windows
   :synopsis: Metafor CIM v1 XML codec.

.. moduleauthor:: Earth System Documentation (ES-DOC) <dev@es-doc.org>

"""
from pyesdoc._codecs.xml_metafor_cim_v1.decoder import decode
from pyesdoc._codecs.xml_metafor_cim_v1.encoder import encode
