{% extends "../core/document.html" %}

{% comment ***************** %}
{% comment Emit content      %}
{% comment ***************** %}
{% block doc-content %}

	{% include activity_ensemble_overview.html %}
	{% if doc.ext.has_changes %}
		{% include activity_ensemble_members.html %}
	{% end %}
	{% if doc.responsible_parties %}
		{% set contacts = doc.responsible_parties %}
		{% set header = 4 %}
		{% include shared_responsible_party.html %}
	{% end %}

{% end %}
