# -*- coding: utf-8 -*-

"""
.. module:: pyesdoc.mp.exceptions
   :platform: Unix, Windows
   :synopsis: Library exceptions.

.. moduleauthor:: Mark Conway-Greenslade <momipsl@ipsl.jussieu.fr>


"""

# Unsupported schema error.
class UnsupportedOntologySchema(ValueError):
    pass