# -*- coding: utf-8 -*-

"""
.. module:: json.__init__.py

   :license: GPL / CeCILL
   :platform: Unix, Windows
   :synopsis: JSON codec.

.. moduleauthor:: Earth System Documentation (ES-DOC) <dev@es-doc.org>

"""
from pyesdoc._codecs.json.decoder import decode
from pyesdoc._codecs.json.encoder import encode
