# -*- coding: utf-8 -*-

"""
.. module:: pyesdoc.cv.codecs.json.__init__.py

   :copyright: @2013 Earth System Documentation (https://es-doc.org)
   :license: GPL / CeCILL
   :platform: Unix, Windows
   :synopsis: Python json codec.

.. moduleauthor:: Earth System Documentation (ES-DOC) <dev@es-doc.org>

"""
from pyesdoc.cv.codecs.json.decoder import decode
from pyesdoc.cv.codecs.json.encoder import encode
