{% if citations %}
<section>
	<header>
		<h{{ header }}>Citations</h{{ header }}>
	</header>
	{% for citation in citations %}
	<section>
		<header>
			<h{{ header + 1 }}>{{ citation.title }}</h{{ header + 1 }}>
		</header>
		<table>
			<tbody>
			{% set ctx = TemplateInfo(citation, fieldset='cim.1.shared.citation') %}
			{% include ../core/fieldset.html %}
			</tbody>
		</table>
	</section>
	{% end %}
</section>
{% end %}
