<section id="{{ doc.ext.full_id }}-details-overview">
	<header>
		<h4>Overview</h4>
	</header>
	<table>
		<tbody>
		{% set ctx = TemplateInfo(doc, fieldset='cim.1.activity.ensemble-overview') %}
		{% include ../core/fieldset.html %}
		</tbody>
	</table>
</section>