<section>
	<header>
		<h{{ header }}>Responsible Parties</h{{ header }}>
	</header>
	{% for responsible in responsible_parties %}
	{% for party in sorted(load(responsible.parties), key=lambda i: i.name.split(" ")[-1]) %}
	{% set ctx = TemplateInfo(party, fieldset='cim.2.shared.party') %}
	<section>
		<header>
			{% if responsible.role %}
			<h{{ header + 1 }}>{{ responsible.role.strip() }}&nbsp;>&nbsp;{{ party.name }}</h{{ header + 1}}>
			{% else %}
			<h{{ header + 1 }}>{{ party.name }}</h{{ header + 1}}>
			{% end %}
		</header>
		<table>
			<tbody>
			{% include ../core/fieldset.html %}
			</tbody>
		</table>
	</section>
	{% end %}
	{% end %}
</section>