# Cardinality constraint.
CONSTRAINT_TYPE_CARDINALITY = 'cardinality'

# Constant constraint.
CONSTRAINT_TYPE_CONSTANT = 'constant'

# Type constraint.
CONSTRAINT_TYPE_TYPEOF = 'type'

# Regular expression constraint.
CONSTRAINT_TYPE_REGEX = 'regex'

# set of supported constraint types.
CONSTRAINT_TYPES = {
	CONSTRAINT_TYPE_CARDINALITY,
	CONSTRAINT_TYPE_CONSTANT,
	CONSTRAINT_TYPE_REGEX,
	CONSTRAINT_TYPE_TYPEOF
}
