
# -*- coding: utf-8 -*-

"""
.. module:: cim.v1.typeset.py

   :license: GPL / CeCILL
   :platform: Unix, Windows
   :synopsis: Typeset information for the cim v1 ontology.

.. moduleauthor:: Earth System Documentation (ES-DOC) <dev@es-doc.org>
.. note:: Code generated using the pyesdoc framework.

"""
from typeset_for_activity_package import *
from typeset_for_data_package import *
from typeset_for_grids_package import *
from typeset_for_misc_package import *
from typeset_for_quality_package import *
from typeset_for_shared_package import *
from typeset_for_software_package import *


import typeset_for_activity_package as activity
import typeset_for_data_package as data
import typeset_for_grids_package as grids
import typeset_for_misc_package as misc
import typeset_for_quality_package as quality
import typeset_for_shared_package as shared
import typeset_for_software_package as software



# Module exports.
__all__ = [
    # Packages
    activity,
    data,
    grids,
    misc,
    quality,
    shared,
    software,
    # Classes
    Activity,
    BoundaryCondition,
    Conformance,
    DownscalingSimulation,
    Ensemble,
    EnsembleMember,
    Experiment,
    ExperimentRelationship,
    ExperimentRelationshipTarget,
    InitialCondition,
    LateralBoundaryCondition,
    MeasurementCampaign,
    NumericalActivity,
    NumericalExperiment,
    NumericalRequirement,
    NumericalRequirementOption,
    OutputRequirement,
    PhysicalModification,
    Simulation,
    SimulationComposite,
    SimulationRelationship,
    SimulationRelationshipTarget,
    SimulationRun,
    SpatioTemporalConstraint,
    DataContent,
    DataDistribution,
    DataExtent,
    DataExtentGeographical,
    DataExtentTemporal,
    DataExtentTimeInterval,
    DataHierarchyLevel,
    DataObject,
    DataProperty,
    DataRestriction,
    DataStorage,
    DataStorageDb,
    DataStorageFile,
    DataStorageIp,
    DataTopic,
    CoordinateList,
    GridExtent,
    GridMosaic,
    GridProperty,
    GridSpec,
    GridTile,
    GridTileResolutionType,
    SimpleGridGeometry,
    VerticalCoordinateList,
    DocumentSet,
    CimQuality,
    Evaluation,
    Measure,
    Report,
    Calendar,
    Change,
    ChangeProperty,
    Citation,
    ClosedDateRange,
    Compiler,
    Daily360,
    DataSource,
    DateRange,
    DocMetaInfo,
    DocReference,
    License,
    Machine,
    MachineCompilerUnit,
    OpenDateRange,
    PerpetualPeriod,
    Platform,
    Property,
    RealCalendar,
    Relationship,
    ResponsibleParty,
    Standard,
    StandardName,
    Component,
    ComponentLanguage,
    ComponentLanguageProperty,
    ComponentProperty,
    Composition,
    Connection,
    ConnectionEndpoint,
    ConnectionProperty,
    Coupling,
    CouplingEndpoint,
    CouplingProperty,
    Deployment,
    EntryPoint,
    ModelComponent,
    Parallelisation,
    ProcessorComponent,
    Rank,
    SpatialRegridding,
    SpatialRegriddingProperty,
    SpatialRegriddingUserMethod,
    StatisticalModelComponent,
    TimeLag,
    TimeTransformation,
    Timing,
    # Enums
    ConformanceType,
    DownscalingType,
    EnsembleType,
    ExperimentRelationshipType,
    FrequencyType,
    ProjectType,
    ResolutionType,
    SimulationRelationshipType,
    SimulationType,
    DataHierarchyType,
    DataStatusType,
    ArcTypeEnum,
    DiscretizationEnum,
    FeatureTypeEnum,
    GeometryTypeEnum,
    GridNodePositionEnum,
    GridTypeEnum,
    HorizontalCsEnum,
    RefinementTypeEnum,
    VerticalCsEnum,
    CimFeatureType,
    CimResultType,
    CimScopeCodeType,
    QualityIssueType,
    QualitySeverityType,
    QualityStatusType,
    ChangePropertyType,
    CompilerType,
    DataPurpose,
    InterconnectType,
    MachineType,
    MachineVendorType,
    OperatingSystemType,
    ProcessorType,
    UnitType,
    ComponentPropertyIntentType,
    ConnectionType,
    CouplingFrameworkType,
    ModelComponentType,
    SpatialRegriddingDimensionType,
    SpatialRegriddingStandardMethodType,
    StatisticalModelComponentType,
    TimeMappingType,
    TimingUnits,
]

